#ifndef H_ARGUMENT_MARKER
# define H_ARGUMENT_MARKER

#include <cassert>
#include <iostream>

namespace ngfem 
{
	typedef struct
	{
		double x;
		double y;
	} ArgumentMarkerData; 

	class ArgumentMarker : public ParameterCoefficientFunction
	{
	};

	class XArgumentMarker : public ArgumentMarker
	{
	public: 
		virtual double Evaluate(const BaseMappedIntegrationPoint& mip) const override
		{
			// doesn't make any sense
			throw Exception("Evaluation not implemented");
		}

		virtual void Evaluate(const BaseMappedIntegrationRule& mir, 
			BareSliceMatrix<double> values) const override
		{
			const ElementTransformation& trafo = mir.GetTransformation();
			ArgumentMarkerData* amd = reinterpret_cast<ArgumentMarkerData*>(trafo.userdata);
			assert(amd != NULL);
			values.AddSize(mir.Size(), 1) = amd->x;
		}
	};
		
	class YArgumentMarker : public ArgumentMarker
	{
	public:
		virtual double Evaluate(const BaseMappedIntegrationPoint& mip) const override
		{
			// doesn't make any sense
			throw Exception("Evaluation not implemented");
		}

		virtual void Evaluate(const BaseMappedIntegrationRule& mir, 
			BareSliceMatrix<double> values) const override
		{
			const ElementTransformation& trafo = mir.GetTransformation();
			ArgumentMarkerData* amd = reinterpret_cast<ArgumentMarkerData*>(trafo.userdata);
			assert(amd != NULL);
			values.AddSize(mir.Size(), 1) = amd->y;
		}
	};

	void ExportArgumentMarkers(py::module m);
}

#endif
